#ifndef CSACFile_h_
#define CSACFile_h_

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <iostream>
using std::cout;
using std::endl;
using std::cerr;

#include <fitsio.h>

extern bool merlin_debug_;

#include "MERLIN_Range.h"
using CSAC::MERLIN_Range;

class CSACFile
{
  
protected:
    ///
    bool _scale_I;
    ///
    bool _scale_Q;
    ///
    bool _scale_U;
    ///
    bool _scale_V;
    ///
    string _data_type;
    ///
    string _filename;
    ///
    bool _open;
    ///
    vector <CSAC::MERLIN_Range> _ranges;
    ///
    fitsfile *_fptr;
    ///
    int _number_slit_positions;
    ///
    int _lambda;
    ///
    void process_status( int &status, string &error )
    {
        char error_description[30]="";
        fits_get_errstatus(status,error_description);
        error=error_description;
    }
    ///
    void real_open();
  
public:
    ///
    CSACFile() : _fptr(0), _open(false),
                 _scale_I(false), _scale_Q(false),
                 _scale_U(false), _scale_V(false) 
    {
    };
    ///
    CSACFile (string &s) : _fptr(0), _open(false), 
                           _scale_I(false), _scale_Q(false),
                           _scale_U(false), _scale_V(false)
    {
        _filename = s;
    }
    ///
    virtual ~CSACFile();
    ///
    virtual void open() 
    {
        real_open();
    }
    ///
    virtual void open(const string & s) 
    {
        if(_open)
            close();
        _filename = s;
        real_open();
    }
    ///
    void close() throw (string);
    ///
    string get_file_name() 
    {
        return _filename;
    }
    ///
    void set_file_name(const string & name) 
    {
        _filename = name;
    }
    ///
    virtual void show_header();
    ///
    virtual int get_number_of_pixels() {
        return _number_slit_positions;
    }
    ///
    virtual int get_size_of_vectors(){
        return _lambda;
    }
    ///
    virtual void get_I(int pos, vector<double> &data);
    ///
    virtual void get_Q(int pos, vector<double> &data);
    ///
    virtual void get_U(int pos, vector<double> &data);
    ///
    virtual void get_V(int pos, vector<double> &data);
    ///
    virtual void get_I(int pos, float *data, const int &len);
    ///
    virtual void get_Q(int pos, float *data, const int &len);
    ///
    virtual void get_U(int pos, float *data, const int &len);
    ///
    virtual void get_V(int pos, float *data, const int &len);
    ///
    virtual void get_inversion_lambda_ranges(vector<MERLIN_Range> &ranges)
    {
        ranges.clear();
        ranges = _ranges;
    }  
    /// 
    virtual void set_inversion_lambda_ranges(const vector<MERLIN_Range> &ranges)
    {
        _ranges.clear();
        _ranges = ranges;
    }
  
};

#endif

