#ifndef FileInversion_h_
#define FileInversion_h_

#include <string>
#include <vector>

#include "CSACFile.h"
#include "InitialGuessFile.h"
#include "MERLIN_Range.h"
#include "MERLIN_Model.h"
#include "MERLIN_Profile.h"
#include "MERLIN_Inversion.h"
// #include "MERLIN_Inversion_SVD.h"
#include "MERLIN_InversionStepWiseRegression.h"


#include "MERLIN_Init.h"


// This protects the compilation
// on Solaris 10 using gcc 3.4.3
#ifdef _B
#undef _B
#endif

using namespace CSAC;

///
class FileInversion
{
    //
    double _minimum_polarization;
    //
    int _n_data;
    //
    std::vector<double> _net_polarization;
    //
    int _net_polarization_id;
    //
    std::vector<double> _unconstrained_net_polarization;
    //
    int _unconstrained_net_polarization_id;
    //
    std::vector<double> _B;
    //
    int _B_id;
    //
    std::vector<double> _B_error;
    //
    int _B_error_id;
    //
    std::vector<double> _Inclination;
    //
    int _Inclination_id;
    //
    std::vector<double> _Inclination_error;
    //
    int _Inclination_error_id;    
    //
    std::vector<double> _Azimuth;
    //
    int _Azimuth_id;
    //
    std::vector<double> _Azimuth_error;
    //
    int _Azimuth_error_id;
    //
    std::vector<double> _Eta0;
    //
    int _Eta0_id;
    //
    std::vector<double> _Eta0_error;
    //
    int _Eta0_error_id;
    //
    std::vector<double> _Damp;
    //
    int _Damp_id;
    //
    std::vector<double> _Damp_error;
    //
    int _Damp_error_id;
    //
    std::vector<double> _Dop;
    //
    int _Dop_id;
    //
    std::vector<double> _Dop_error;
    //
    int _Dop_error_id;
    //
    std::vector<double> _B0;
    //
    int _B0_id;
    //
    std::vector<double> _B0_error;
    //
    int _B0_error_id;
    //
    std::vector<double> _B1;
    //
    int _B1_id;
    //
    //
    std::vector<double> _B1_error;
    //
    int _B1_error_id;
    //
    std::vector<double> _vmac;
    //
    int _vmac_id;
    //
    std::vector<double> _vmac_error;
    //
    int _vmac_error_id;
    //
    std::vector<double> _lines_shift1;
    //
    int _lines_shift1_id;
    //
    std::vector<double> _lines_shift1_error;
    //
    int _lines_shift1_error_id;
    //
    std::vector<double> _lines_shift2;
    //
    int _lines_shift2_id;
    //
    std::vector<double> _lines_shift2_error;
    //
    int _lines_shift2_error_id;
    //
    std::vector<double> _stray_light;
    //
    int _stray_light_id;
    //
    //
    std::vector<double> _stray_light_error;
    //
    int _stray_light_error_id;
    //
    std::vector<double> _stray_light_shift;
    //
    int _stray_light_shift_id;
    //
    std::vector<double> _stray_light_shift_error;
    //
    int _stray_light_shift_error_id;
    //
    std::vector<double> _chisq;
    //
    int _chisq_id;
    //
    std::vector<double> _chisq_I;
    //
    int _chisq_I_id;
    //
    std::vector<double> _chisq_Q;
    //
    int _chisq_Q_id;
    //
    std::vector<double> _chisq_U;
    //
    int _chisq_U_id;
    //
    std::vector<double> _chisq_V;
    //
    int _chisq_V_id;
    //
    std::vector<double> _signed_stokes_V;
    //
    int _signed_stokes_V_id;
    //
    std::vector<double> _n_iterations;
    //
    int _n_iterations_id;
    // Theoretical continuum
    std::vector <double> _continuum;
    int _continuum_id;
    // Observed continuum
    std::vector <double> _observed_continuum;
    int _observed_continuum_id;
    //
    std::vector <double> _reason_for_convergence;
    int _rconv_id;
    //
    std::vector<double> _B_init;
    int _B_init_id;
    //
    std::vector<double> _Inclination_init;
    int _Inclination_init_id;
    //
    std::vector<double> _Azimuth_init;
    int _Azimuth_init_id;
    //
    std::vector<double> _Eta0_init;
    int _Eta0_init_id;
    //
    std::vector<double> _Damp_init;
    int _Damp_init_id;
    //
    std::vector<double> _Dop_init;
    int _Dop_init_id;
    //
    std::vector<double> _B0_init;
    int _B0_init_id;
    //
    std::vector<double> _B1_init;
    int _B1_init_id;
    //
    std::vector<double> _vmac_init;
    int _vmac_init_id;
    //
    std::vector<double> _lines_shift1_init;
    ///
    int _lines_shift1_init_id;
    //
    std::vector<double> _lines_shift2_init;
    int _lines_shift2_init_id;
    //
    std::vector<double> _stray_light_init;
    int _stray_light_init_id;
    //
    std::vector<double> _stray_light_shift_init;
    //
    int _stray_light_shift_init_id;
    //
    int _out_ncid;
    //
    int _dim_ids[1];
    //
    int _stokes_I_id;
    //
    int _stokes_Q_id;
    //
    int _stokes_U_id;
    //
    int _stokes_V_id;
    //
    CSACFile *_file;
    //
    InitialGuessFile * _guess_file;
    ///
    MERLIN_StokesProfile observed;
    //
    MERLIN_Model _par;
    //
    MERLIN_Inversion *_inverter;
    //
    //
    vector <MERLIN_Range> ranges;
    //
    bool _profile_dumper_set;
    //
    int _observed_id;
    //
    int _synthetic_id;
    //
    int _lambda_id;
    //
    void write_profiles_to_dumper(const MERLIN_StokesProfile &, const MERLIN_StokesProfile &, int );
    //
    void set_observed_profile_by_ranges();
    //
    float get_net_polarization_of_full_spectrum();
    //
    double get_net_polarization_of_constrained_spectrum();
    //
    void call_inversion_kernel(int, int&, double&);
    //
    //string load_spectral_grid_name();
    //
    void prime_netCDF_file(const string &outfile);
    //
    void close_netCDF_file();
    //
    double self_initialize_pixel(double&);
    //
    int _size_of_vectors;
    //
    float *_stokes_I;
    //
    float *_stokes_Q;
    //
    float *_stokes_U;
    //
    float *_stokes_V;
    //
    float _Imax;
    //
    void clean_memory();
    // 
    double compute_magnitude_stokes_V();
public:
    ///
    int individuals;
    ///
    int generations;
    ///	
    int Y_min;
    ///
    int Y_max;
    ///
    FileInversion();
    ///
    ~FileInversion();
    ///
    void run(CSACFile *, InitialGuessFile *, const string &) throw(string, CSACFile);
    ///
    friend class CSACFile;
    ///
    void set_file_dumper() {_profile_dumper_set = true; }
    ///
    void set_model(const MERLIN_Model &model) {_par = model;}
    ///
    void set_minimum_polarization(double &mp){ _minimum_polarization = mp;}
    ///
    friend void MERLIN_Init::create_initial_guess(
                                                  float * I,
                                                  float * Q,
                                                  float * U,
                                                  float * V,
                                                  int &len,
                                                  CSAC::MERLIN_StokesProfile &observed,
                                                  float &continuum, 
                                                  CSAC::MERLIN_Model &my_model,
                                                  double &genetic_chisq,
                                                  double &netpol,
                                                  int &individuals,
                                                  int &generations
                                                  );
    void set_inverter(string &inverter_type) throw (string)
    {
        if (_inverter) 
            delete _inverter;
	/*
        if (inverter_type == "SVD")
            _inverter = new MERLIN_Inversion_SVD();
	*/
        if (inverter_type == "SWR")
            _inverter = new MERLIN_InversionStepWiseRegression();
        else
            throw string("The requested type of inverter is invalid for the class  FileInversion.");
    }
            
	
};


#endif // FileInversion_h_
