/*
 *  InitialGuessFilenetCDF.cc
 *  fico
 *
 *  Created by Jose Garcia on 8/17/06.
 *  Copyright 2006 NCAR/HAO. All rights reserved.
 *
 */

#include <iostream>

#include <string.h>
#include <stdlib.h>

#include "InitialGuessFilenetCDF.h"

#include "netcdf.h"

using namespace CSAC;

extern bool merlin_client_debug_;

InitialGuessFilenetCDF::~InitialGuessFilenetCDF()
{
	ncopts = NC_VERBOSE;
	if (_ncid_set)
	{
		int status = nc_close(_ncid);
		if (status != NC_NOERR) 
		{
			cout<<__FILE__<<":"<<__LINE__<<": ERROR!"<<endl;
			nc_advise("nc_close",status,"");
			exit(1);
		}
	}
	
}


void 
InitialGuessFilenetCDF::load_contents() throw (string)
{
	// attempt to load all variables, if anyone fails rethrow exception
	try 
	{
		load_variable("SFLD");
		//load_variable("RUFF_SFLD");
		load_variable("SPSI");
		load_variable("SAZM");
		load_variable("SETA0");
		load_variable("SDMP");
		load_variable("SDOP");
		load_variable("SB1MU");
		load_variable("SBZERO");
		load_variable("SALPHA");
		load_variable("SDELTA");
		// now we load the velocity shift for the spectral lines
		nc_type nlines_type;
		size_t nlines_len;
		int status = nc_inq_att (_ncid, NC_GLOBAL, "nlines", &nlines_type, &nlines_len);
		if (status != NC_NOERR)
		{
			string error = string("while attempting to inquire about attribute nlines, the netcdf API returs: ") + string(nc_strerror(status));
			throw error;
		}
		if( nlines_type != NC_SHORT)
		{
			string error = string("Attribute nlines is not of type NCSHORT");
			throw error;
		}
		short nlines_val;
		status = nc_get_att_short(_ncid,NC_GLOBAL,"nlines", &nlines_val);
		if (status != NC_NOERR)
		{
			string error = string("while attempting to obtain the value for attribute nlines, the netcdf API returs: ") + string(nc_strerror(status));
			throw error;
		}
		if (merlin_client_debug_)
			cout<<"The number of lines in this file is "<<nlines_val<<endl;
		if (nlines_val != 2)
		{
			string error ("The number of spectral lines is not two, this client can only extract data from netcdf files with two lines");
			throw error;
		}
		vector <float> my_tmp_vec;
		my_tmp_vec.resize(1,0.0);
		// push two vectors line this one to hold the shift for the two lines
		_lines_shift.push_back(my_tmp_vec);
		_lines_shift.push_back(my_tmp_vec);
		load_variable("SCEN2");
		load_variable("SCEN1");
		
	}
	catch (string &str)
	{
		throw;
	}
}

bool 
InitialGuessFilenetCDF::open() throw()
{
	ncopts = 0;
	int status = nc_open(_name.c_str(), 0, &_ncid);
	if (status != NC_NOERR) 
		return false;
	
	_ncid_set = true;
	
	return true;
}

void 
InitialGuessFilenetCDF::load_variable(const string &variable) throw (string)
{
	if (merlin_client_debug_)
		cout<<"Loading initial guess for variable "<<variable<<endl;
	int ndims;
	int var_id;
	int status = nc_inq_varid (_ncid, variable.c_str(), &var_id);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	status = nc_inq_varndims (_ncid, var_id, &ndims);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	if(ndims != 1)
	{
		string error = string("the number of dimensions for variable ") + variable + string(" is not one");
		throw error;
	}
	int dimensions_ids[1];
	status = nc_inq_vardimid (_ncid, var_id, dimensions_ids);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	char recname[NC_MAX_NAME+1];
	status = nc_inq_dimname (_ncid, dimensions_ids[0], recname);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	size_t len;
	status = nc_inq_dimlen  (_ncid, dimensions_ids[0], &len);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	nc_type var_type;
	status = nc_inq_vartype(_ncid, var_id, &var_type);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	if (var_type != NC_FLOAT)
	{
		string error = string("the variable ") + variable + string(" is not of type NC_FLOAT");
		throw error;
	}
	float *var_data;
	var_data = new float [len];
	status = nc_get_var_float(_ncid, var_id, var_data);
	if (status != NC_NOERR)
	{
		string error = string("while attempting to load variable ") + variable + string(", the netcdf API returs: ") + string(nc_strerror(status));
		throw error;
	}
	if (variable == "SFLD")
	{
		_B.resize(len, 0.0);
		memcpy(&_B[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "RUFF_SFLD")
	{
		_ruff_B.resize(len, 0.0);
		memcpy(&_ruff_B[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SPSI")
	{
		_Inclination.resize(len, 0.0);
		memcpy(&_Inclination[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SAZM")
	{
		_Azimuth.resize(len, 0.0);
		memcpy(&_Azimuth[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SETA0")
	{
		_Eta0.resize(len, 0.0);
		memcpy(&_Eta0[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SDMP")
	{
		_Damp.resize(len, 0.0);
		memcpy(&_Damp[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SDOP")
	{
		_Dop.resize(len, 0.0);
		memcpy(&_Dop[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SB1MU")
	{
		_B1.resize(len, 0.0);
		memcpy(&_B1[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SBZERO")
	{
		_B0.resize(len, 0.0);
		memcpy(&_B0[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SALPHA")
	{
		_stray_light.resize(len, 0.0);
		memcpy(&_stray_light[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SDELTA")
	{
		_stray_light_shift.resize(len, 0.0);
		memcpy(&_stray_light_shift[0], var_data, (len * sizeof(float)) );
	}
	else if (variable == "SCEN2")
	{
            // Notice that according to the ASP definition
            // SCEN2 is the doppler shift to 6301.5, thuse
            // we are adding SCEN2 to _lines_shift[0] 
            // and we are adding SCEN1 to _lines_shift[1]
            // in the next 'else if' case
		_lines_shift[0].resize(len, 0.0);
		memcpy(&(_lines_shift[0][0]), var_data, (len * sizeof(float)) );
	}
	else if (variable == "SCEN1")
	{
		_lines_shift[1].resize(len, 0.0);
		memcpy(&(_lines_shift[1][0]), var_data, (len * sizeof(float)) );
	}
	else
        {
            string error = string("Invalid variable name: ") + variable; 
            throw error;
        }

	delete [] var_data;
}
