/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ConvolveProfile.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#ifndef MERLIN_ConvolveProfile_h_
#define MERLIN_ConvolveProfile_h_ 1

/** \file convolve_profile.h 
    Contains the global function to convolve profiles with macroturbulence.
*/

namespace CSAC
{

    // Foward declarations
    class MERLIN_Model;
    class MERLIN_Profile;

    /// This method convolves a profile with the macroturbulence, assuming that 
    /// the macroturbulence is in the form of a Gaussian.
    /// If the last parameter is false, only the profile convolved.
    /// If the last parameter is true, then the profile and its derivatives are convolved.
    void MERLIN_ConvolveProfile(MERLIN_Model &, MERLIN_Profile &, bool f);

} // end namespace CSAC

#endif // MERLIN_ConvolveProfile_h_
