/* ============================================================================
* Language: C++ 
* File: MERLIN_Inversion.h
* Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
* Sub-package: Fundamental Inversion Code (FICO).
*
* Created by Jose Garcia <jgarcia@ucar.edu> on 08/13/08.
*  
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* You can contact University Corporation for Atmospheric Research at
* 3080 Center Green Drive, Boulder, CO 80301

* (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
* Please read the full copyright statement in the file COPYRIGHT_UCAR.
*
=========================================================================== */

#ifndef MERLIN_Inversion_h_
#define MERLIN_Inversion_h_ 1

#include <math.h>

#include "MERLIN_Object.h"
#include "MERLIN_ParametersLimits.h"

namespace CSAC
{

    class MERLIN_Inversion : public MERLIN_Object
    {
    public:
        /// Limits for this inversion object
        MERLIN_ParametersLimits limits_for_inversion;
         /// Destructors of parent classes must always be virtual.
        virtual ~MERLIN_Inversion() {}
        /// This tell us why the the inversion stopped
        enum exit_condition {UNSET_INVERSION_NOT_PERFORMED =0, 
                             REACHED_MAX_ITERATIONS, 
                             REACHED_MAX_FAILURES, 
                             REACH_CONVERGENCE_CRITERIA, 
                             LAMBDA_GREATER_THAM_MAX_ALLOWED, 
                             LAMBDA_SMALLER_THAM_MIN_ALLOWED, 
                             NOTHING_TO_DO};
        ///
        virtual void set_continuum_normalization(const double &v) = 0; 
        /// change the default number of maximun iterations
        virtual void set_maximun_iterations (const int &max) = 0; 
        /// return the reason why the inversion stopped
        virtual exit_condition get_exit_condition() = 0; 
        /// set the maximum damping parameter
        virtual void set_max_damping_parameter(const double & d) = 0;
        /// set the minimum damping parameter
        virtual void set_min_damping_parameter(const double & d) = 0; 
        /// set maximum number of consecutive failures 
        virtual void set_max_number_failures(const int & f) = 0;
        /// set the worst condition number for the matrix A
        virtual void set_worst_tolerated_condition_number(const double & d) = 0;
        /// Pass to the MERLIN_SVDSolver member the threshold.
        // virtual void set_svd_threshold(const double &pass_along_d) = 0;
        ///
        virtual void set_max_planck_function(double &val) = 0;
        ///
        virtual void set_max_planck_function_gradient(double &val) = 0;
        /// Printer method, required by MERLIN_Object.
        //virtual void dump( ostream &strm ) const = 0;
         ///
        virtual double execute (MERLIN_StokesProfile &observed_profile, 
                                MERLIN_Model &guess, 
                                MERLIN_StokesProfile *sigma, 
                                int &n_iterations,
                                double &chisq_I,
                                double &chisq_U,
                                double &chisq_Q,
                                double &chisq_V) = 0;

    }; // end class MERLIN_Inversion

} // end namespace CSAC
#endif //  MERLIN_Inversion__h
