/* ============================================================================
 * Language: C++ 
 * File: MERLIN_MVoigt.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef mvoigt_h_
#define mvoigt_h_ 1

#include <vector>
using std::vector;

#include "MERLIN_ZeemanComponent.h"
using CSAC::MERLIN_ZeemanComponent;

namespace CSAC{
  
    /// \brief Produce the broadening for the lines Psi anf Phi (Pi, Red and blue).
    void mvoigt(
                vector<MERLIN_ZeemanComponent> comp,
                double a,                     /* damping */
                double Delta_lambda,          /* displacement on Voigt */
                double H_doppler,             /* magnitude of the magnetic field divided by the width of the doppler line */
                double &Phi_p,              /* eta para luz circular derecha */
                double &Psi_p,              /* perfil anomalo */
                double &Phi_r,     /* eta para luz circular derecha */
                double &Psi_r,     /* perfil anomalo */
                double &Phi_b,      /* eta para luz circular izquierda */
                double &Psi_b       /* perfil anomalo */
                );
    /// \brief Produce the broadening for the lines Psi anf Phi (Pi, Red and blue) as well as their devivatives.	
    void mvoigt_d( 
                  vector<MERLIN_ZeemanComponent> comp,
                  double a,                          /* damping */
                  double Delta_lambda,               /* displacement on Voigt */
                  double H_doppler,                  /* magnitude of the magnetic field divided by the width of the doppler line */
                  double &Phi_p,                     /* eta para luz circular derecha */
                  double &Psi_p,                     /* perfil anomalo */
                  double &Phi_r,                     /* eta para luz circular derecha */
                  double &Psi_r,                     /* perfil anomalo */
                  double &Phi_b,                     /* eta para luz circular izquierda */
                  double &Psi_b,                     /* perfil anomalo */
                  double dH_doppler__dB,
                  double dH_doppler__dDldop,
                  double dDelta_lambda__dDldop,
                  double dDelta_lambda__dv,
                  double & dPhi_p__dv,
                  double & dPhi_r__dv,
                  double & dPhi_b__dv,
                  double & dPhi_p__dB,
                  double & dPhi_r__dB,
                  double & dPhi_b__dB,
                  double & dPhi_p__da,
                  double & dPhi_r__da,
                  double & dPhi_b__da,
                  double & dPhi_p__dDldop,
                  double & dPhi_r__dDldop,
                  double & dPhi_b__dDldop,
                  double & dPsi_p__dB,
                  double & dPsi_b__dB,
                  double & dPsi_r__dB,
                  double & dPsi_p__dv,
                  double & dPsi_r__dv,
                  double & dPsi_b__dv,
                  double & dPsi_p__da,
                  double & dPsi_b__da,
                  double & dPsi_r__da,
                  double & dPsi_p__dDldop,
                  double & dPsi_b__dDldop,
                  double & dPsi_r__dDldop
                   );
  
}; // end namespace CSAC

#endif // mvoigt_h_

