/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Region.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_Region_h_
#define MERLIN_Region_h_ 1

#include <iostream>
using std::endl;

#include "MERLIN.h"

namespace CSAC
{
  
#include "MERLIN_Object.h"  
    using CSAC::MERLIN_Object;
  
    ///  Simple class to hold together the data that defines a region of the spectrum.
    class MERLIN_Region : public MERLIN_Object
    {
    public :
    
        // First_wavelength in A
        merlin_real first_wavelength ;
        // Wave_step is in mA 
        merlin_real wave_step ;
        // Number of waves for this region
        int n_wavelengths ;
    public :
        // Default constructor 
        MERLIN_Region() : 
            first_wavelength( 0.0 ),
            wave_step( 0.0 ),
            n_wavelengths( 0 ) 
        {}
        //
        MERLIN_Region(const MERLIN_Region &rhs)
        {
            // First_wavelength in A
            first_wavelength = rhs.first_wavelength ;
            // Wave_step is in mA 
            wave_step = rhs.wave_step ;
            // Number of waves for this region
            n_wavelengths = rhs.n_wavelengths ;
        }
        // Destructor
        virtual ~MERLIN_Region() {};
        /// Printer method
        virtual void dump( ostream &strm ) const 
        {
            strm << "        Region:" << endl ;
            strm << "            First_wavelength = "<< first_wavelength<< endl ;
            strm << "            Wave_step = " << wave_step<< endl ;
            strm << "            n_wavelengths = " << n_wavelengths<< endl ;
        }
        /// Overloaded = operator.
        MERLIN_Region& operator=(const MERLIN_Region &rhs)
        {
            if(this == &rhs)
                return *this;
            // First_wavelength in A
            first_wavelength = rhs.first_wavelength ;
            // Wave_step is in mA 
            wave_step = rhs.wave_step ;
            // Number of waves for this region
            n_wavelengths = rhs.n_wavelengths ;
            return *this;
        }
    
    }; //end class MERLIN_Region
  
} // end namespace CSAC

#endif // MERLIN_Region_h_
