/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ZeemanComponent.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#ifndef MERLIN_ZeemanComponent_h_
#define MERLIN_ZeemanComponent_h_ 1

#include <iostream>
using std::ostream;
using std::endl;

#include "MERLIN_Object.h"
using CSAC::MERLIN_Object;

#include "MERLIN.h"

namespace CSAC
{
  
    /** \file MERLIN_ZeemanComponent.h
     */
  
    /// \brief Simple class to hold a Zeeman component caused by Zeeman Splitting.
    class MERLIN_ZeemanComponent : public MERLIN_Object
    {
    public:
  
        /// Type of Zeeman component, which can be PI, SIGMA_RIGHT, SIGMA_LEFT
        typedef enum MERLIN_ZeemanComponentType_ { PI=0, SIGMA_RIGHT, SIGMA_LEFT} MERLIN_ZeemanComponentType;
	/// Component type.
	/// The type of component can be PI, SIGMA_RIGHT, SIGMA_LEFT
	/// @see enum MERLIN_ZeemanComponent_Type_
	MERLIN_ZeemanComponentType type;
        /// See the definition of strenght in PDF file.
	merlin_real strength;
	/// See the definition of derivative in  PDF file.
	merlin_real derivative;
  
    public :
        /// Default constructor
        MERLIN_ZeemanComponent() :
            type(PI),
            strength(0.0),
            derivative(0.0)
        {}
	/// Copy constructor
	MERLIN_ZeemanComponent(const MERLIN_ZeemanComponent &rhs)
        {
            type=rhs.type;
            strength=rhs.strength;
            derivative=rhs.derivative;
        }  
	/// Destructor
	virtual ~MERLIN_ZeemanComponent() {}
	/// Overloaded = operator.
	MERLIN_ZeemanComponent& operator=(const MERLIN_ZeemanComponent &rhs)
        {
            if(this == &rhs)
                return (*this);
            type=rhs.type;
            strength=rhs.strength;
            derivative=rhs.derivative;
            return *this;
        }
	/// Printer method.
	virtual void dump( ostream &strm ) const 
        {
            strm<<"Component Type "<<type<<endl;
            strm<<"Component Strength "<<strength<<endl;
            strm<<"Component Derivative "<<derivative<<endl;
        }
    }; // end class MERLIN_ZeemanComponent

} // end namespace CSAC



#endif // MERLIN_MERLIN_ZeemanComponent_h_
